@@ -9,12 +9,9 @@ import pytest
 
 from xonsh.ast import AST, With, Pass, Str, Call
 from xonsh.parser import Parser
-from xonsh.parsers.base import eval_fstr_fields
+from xonsh.parsers.fstring_adaptor import FStringAdaptor
 
-from tools import VER_FULL, skip_if_lt_py36, nodes_equal
-
-# a lot of col_offset data changed from Py v3.5.0 -> v3.5.1
-INC_ATTRS = (3, 5, 1) <= VER_FULL
+from tools import nodes_equal, skip_if_no_walrus, VER_MAJOR_MINOR
 
 
 @pytest.fixture(autouse=True)
@@ -77,7 +74,6 @@ def test_int_literal():
     check_ast("42")
 
 
-@skip_if_lt_py36
 def test_int_literal_underscore():
     check_ast("4_2")
 
@@ -86,7 +82,6 @@ def test_float_literal():
     check_ast("42.0")
 
 
-@skip_if_lt_py36
 def test_float_literal_underscore():
     check_ast("4_2.4_2")
 
@@ -117,41 +112,58 @@ def test_raw_literal():
     check_ast('R"hell\\o"')
 
 
-@skip_if_lt_py36
 def test_f_literal():
     check_ast('f"wakka{yo}yakka{42}"', run=False)
     check_ast('F"{yo}"', run=False)
 
 
-@skip_if_lt_py36
 def test_f_env_var():
     check_xonsh_ast({}, 'f"{$HOME}"', run=False)
     check_xonsh_ast({}, "f'{$XONSH_DEBUG}'", run=False)
     check_xonsh_ast({}, 'F"{$PATH} and {$XONSH_DEBUG}"', run=False)
 
 
-@pytest.mark.parametrize(
-    "inp, exp",
-    [
-        ('f"{}"', 'f"{}"'),
-        ('f"$HOME"', 'f"$HOME"'),
-        ('f"{0} - {1}"', 'f"{0} - {1}"'),
-        (
-            'f"{$HOME}"',
-            "f\"{__xonsh__.execer.eval(r'$HOME', glbs=globals(), locs=locals())}\"",
-        ),
-        (
-            'f"{ $HOME }"',
-            "f\"{__xonsh__.execer.eval(r'$HOME ', glbs=globals(), locs=locals())}\"",
-        ),
-        (
-            "f\"{'$HOME'}\"",
-            "f\"{__xonsh__.execer.eval(r'\\'$HOME\\'', glbs=globals(), locs=locals())}\"",
-        ),
-    ],
-)
-def test_eval_fstr_fields(inp, exp):
-    obs = eval_fstr_fields(inp, 'f"')
+fstring_adaptor_parameters = [
+    ('f"$HOME"', "$HOME"),
+    ('f"{0} - {1}"', "0 - 1"),
+    ('f"{$HOME}"', "/foo/bar"),
+    ('f"{ $HOME }"', "/foo/bar"),
+    ("f\"{'$HOME'}\"", "$HOME"),
+    ('f"$HOME  = {$HOME}"', "$HOME  = /foo/bar"),
+    ("f\"{${'HOME'}}\"", "/foo/bar"),
+    ("f'{${$FOO+$BAR}}'", "/foo/bar"),
+    ("f\"${$FOO}{$BAR}={f'{$HOME}'}\"", "$HOME=/foo/bar"),
+    (
+        '''f"""foo
+{f"_{$HOME}_"}
+bar"""''',
+        "foo\n_/foo/bar_\nbar",
+    ),
+    (
+        '''f"""foo
+{f"_{${'HOME'}}_"}
+bar"""''',
+        "foo\n_/foo/bar_\nbar",
+    ),
+    (
+        '''f"""foo
+{f"_{${ $FOO + $BAR }}_"}
+bar"""''',
+        "foo\n_/foo/bar_\nbar",
+    ),
+]
+if VER_MAJOR_MINOR >= (3, 8):
+    fstring_adaptor_parameters.append(("f'{$HOME=}'", "$HOME='/foo/bar'"))
+
+
+@pytest.mark.parametrize("inp, exp", fstring_adaptor_parameters)
+def test_fstring_adaptor(inp, exp):
+    joined_str_node = FStringAdaptor(inp, "f").run()
+    assert isinstance(joined_str_node, ast.JoinedStr)
+    node = ast.Expression(body=joined_str_node)
+    code = compile(node, "<test_fstring_adaptor>", mode="eval")
+    builtins.__xonsh__.env = {"HOME": "/foo/bar", "FOO": "HO", "BAR": "ME"}
+    obs = eval(code)
     assert exp == obs
 
 
@@ -315,7 +327,7 @@ def test_in():
 
 
 def test_is():
-    check_ast("42 is 65")
+    check_ast("int is float")  # avoid PY3.8 SyntaxWarning "is" with a literal
 
 
 def test_not_in():
@@ -323,7 +335,7 @@ def test_not_in():
 
 
 def test_is_not():
-    check_ast("42 is not 65")
+    check_ast("float is not int")
 
 
 def test_lt_lt():
@@ -781,6 +793,10 @@ def test_listcomp_if_and():
     check_ast('[x for x in "mom" if True and x == "m"]')
 
 
+def test_listcomp_multi_if():
+    check_ast('[x for x in "mom" if True if x in "mo" if x == "m"]')
+
+
 def test_dbl_listcomp():
     check_ast('[x+y for x in "mom" for y in "dad"]')
 
@@ -1193,6 +1209,16 @@ def test_rshift_op_three():
     check_ast("42 >> 65 >> 1 >> 7")
 
 
+@skip_if_no_walrus
+def test_named_expr():
+    check_ast("(x := 42)")
+
+
+@skip_if_no_walrus
+def test_named_expr_list():
+    check_ast("[x := 42, x + 1, x + 2]")
+
+
 #
 # statements
 #
@@ -1338,7 +1364,6 @@ def test_equals_attr():
     check_stmts("class X(object):\n  pass\nx = X()\nx.a = 65")
 
 
-@skip_if_lt_py36
 def test_equals_annotation():
     check_stmts("x : int = 42")
 
@@ -2008,6 +2033,26 @@ def test_async_await():
     check_stmts("async def f():\n    await fut\n", False)
 
 
+@skip_if_no_walrus
+def test_named_expr_args():
+    check_stmts("id(x := 42)")
+
+
+@skip_if_no_walrus
+def test_named_expr_if():
+    check_stmts("if (x := 42) > 0:\n  x += 1")
+
+
+@skip_if_no_walrus
+def test_named_expr_elif():
+    check_stmts("if False:\n  pass\nelif x := 42:\n  x += 1")
+
+
+@skip_if_no_walrus
+def test_named_expr_while():
+    check_stmts("y = 42\nwhile (x := y) < 43:\n  y += 1")
+
+
 #
 # Xonsh specific syntax
 #
