@@ -363,8 +363,8 @@ class XonshKernel:
                 "payload": [],
                 "user_expressions": {},
             }
-        shell = builtins.__xonsh_shell__
-        hist = builtins.__xonsh_history__
+        shell = builtins.__xonsh__.shell
+        hist = builtins.__xonsh__.history
         try:
             shell.default(code, self, parent_header)
             interrupted = False
@@ -421,7 +421,7 @@ class XonshKernel:
 
     def do_complete(self, code, pos):
         """Get completions."""
-        shell = builtins.__xonsh_shell__
+        shell = builtins.__xonsh__.shell
         line = code.split("\n")[-1]
         line = builtins.aliases.expand_alias(line)
         prefix = line.split(" ")[-1]
@@ -463,16 +463,16 @@ class XonshKernel:
 if __name__ == "__main__":
     setup(
         shell_type="jupyter",
-        env={'PAGER': 'cat'},
-        aliases={'less': 'cat'},
-        xontribs=['coreutils'],
-        threadable_predictors={'git': predict_true, 'man': predict_true},
+        env={"PAGER": "cat"},
+        aliases={"less": "cat"},
+        xontribs=["coreutils"],
+        threadable_predictors={"git": predict_true, "man": predict_true},
     )
-    if builtins.__xonsh_commands_cache__.is_only_functional_alias('cat'):
+    if builtins.__xonsh__.commands_cache.is_only_functional_alias("cat"):
         # this is needed if the underlying system doesn't have cat
         # we supply our own, because we can
-        builtins.aliases['cat'] = 'xonsh-cat'
-        builtins.__xonsh_env__['PAGER'] = 'xonsh-cat'
-    shell = builtins.__xonsh_shell__
+        builtins.aliases["cat"] = "xonsh-cat"
+        builtins.__xonsh__.env["PAGER"] = "xonsh-cat"
+    shell = builtins.__xonsh__.shell
     kernel = shell.kernel = XonshKernel()
     kernel.start()
